'''
Copyright (C) 2023

Created by AlexeyHRDesign

'''

# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####


import bpy
from .register import register_class, unregister_class
import os
import time
import json


# Get the path to the current file (addon script)
addon_dir = os.path.dirname(__file__)
# The path to the preset file
preset_file_path = os.path.join(addon_dir, "material_presets.json")

# Check if the file exists
if not os.path.exists(preset_file_path) or os.stat(preset_file_path).st_size == 0:
    # Если файл не существует или пуст, инициализируем его с пустым объектом
    with open(preset_file_path, 'w') as outfile:
        json.dump({}, outfile)

# Safely download the data from the fileа
with open(preset_file_path, 'r') as infile:
    presets = json.load(infile)

blender_version = bpy.app.version


# Option to check if the Base Color texture is present
check_basecolor_texture=True

if blender_version[0] >= 4:
    textures_info={
        "basecolor":{"color_space":"sRGB","display_name":"Basecolor"},
        "normal":{"color_space":"Non-Color","display_name":"Normal"},
        "roughness":{"color_space":"Non-Color","display_name":"Roughness"},
        "metallic":{"color_space":"Non-Color","display_name":"Metallic"},
        "opacity":{"color_space":"Non-Color","display_name":"Alpha"},
        "displacement":{"color_space":"Non-Color","display_name":"Displacement"},
        "emission":{"color_space":"sRGB","display_name":"Emission Color"},
        "ao":{"color_space":"Non-Color","display_name":"AO"},
        "subsurface":{"color_space":"sRGB","display_name":"Subsurface Weight"},
        # "subsurface_radius":{"color_space":"sRGB", "display_name": "Base Color"},
        "specular":{"color_space":"Non-Color","display_name":"Specular IOR Lvl"},
        "specular_tint":{"color_space":"Non-Color","display_name":"Specular Tint"},
        "anisotropic":{"color_space":"Non-Color","display_name":"Anisotropy"},
        "sheen":{"color_space":"Non-Color","display_name":"Sheen Weight"},
        "sheen_tint":{"color_space":"Non-Color","display_name":"Sheen Tint"},
        "clearcoat":{"color_space":"Non-Color","display_name":"Coat Weight"},
        "clearcoat_roughness":{"color_space":"Non-Color","display_name":"Coat Roughness"},
        "ior":{"color_space":"Non-Color","display_name":"IOR"},
        "transmission":{"color_space":"Non-Color","display_name":"Transmission Weight"},
    }
else:
    textures_info={
        "basecolor":{"color_space":"sRGB", "display_name": "Basecolor"},
        "normal":{"color_space":"Non-Color", "display_name": "Normal"},
        "roughness":{"color_space":"Non-Color", "display_name": "Roughness"},
        "metallic":{"color_space":"Non-Color", "display_name": "Metallic"},
        "opacity":{"color_space":"Non-Color", "display_name": "Alpha"},
        "displacement":{"color_space":"Non-Color", "display_name": "Displacement"},
        "emission":{"color_space":"sRGB", "display_name": "Emission"},
        "ao":{"color_space":"Non-Color", "display_name": "AO"},
        "subsurface":{"color_space":"sRGB", "display_name": "Subsurface"},
        # "subsurface_radius":{"color_space":"sRGB", "display_name": "Base Color"},
        "specular":{"color_space":"Non-Color", "display_name": "Specular"},
        "specular_tint":{"color_space":"Non-Color", "display_name": "SpecularTint"},
        "anisotropic":{"color_space":"Non-Color", "display_name": "Anisotropy"},
        "sheen":{"color_space":"Non-Color", "display_name": "Sheen"},
        "sheen_tint":{"color_space":"Non-Color", "display_name": "SheenTint"},
        "clearcoat":{"color_space":"Non-Color", "display_name": "Coat"},
        "clearcoat_roughness":{"color_space":"Non-Color", "display_name": "CoatRoughness"},
        "ior":{"color_space":"Non-Color", "display_name": "IOR"},
        "transmission":{"color_space":"Non-Color", "display_name": "Transmission"},
    }

# Define available Colorspaces
if blender_version[0] >= 4:
    colorspace_items=[
        ("Filmic Log","Filmic Log","Filmic Log"),
        ("Filmic sRGB","Filmic sRGB","Filmic sRGB"),
        ("Linear","Linear","Linear"),
        ("Linear ACES","Linear ACES","Linear ACES"),
        ("Linear ACEScg","Linear ACEScg","Linear ACEScg"),
        ("Non-Color","Non-Color","Non-Color"),
        ("sRGB","sRGB","sRGB"),
    ]
else:
    colorspace_items=[
        ("Filmic Log","Filmic Log","Filmic Log"),
        ("Filmic sRGB","Filmic sRGB","Filmic sRGB"),
        ("Linear","Linear","Linear"),
        ("Linear ACES","Linear ACES","Linear ACES"),
        ("Linear ACEScg","Linear ACEScg","Linear ACEScg"),
        ("Non-Color","Non-Color","Non-Color"),
        ("Raw","Raw","Raw"),
        ("sRGB","sRGB","sRGB"),
        ("XYZ","XYZ","XYZ"),
    ]

image_formats={
    "PNG":"PNG",
    "JPEG":"JPEG",
    "JPG":"JPG",
    "TIFF":"TIFF",
    # Add new formats here if needed
}

# Definition of blend_modes
blend_modes = [
    ('OPAQUE', "Opaque", "Opaque Blend Mode"),
    ('CLIP', "Alpha Clip", "Alpha Clip Blend Mode"),
    ('HASHED', "Alpha Hashed", "Alpha Hashed Blend Mode"),
    ('BLEND', "Alpha Blend", "Alpha Blend Blend Mode")
]

bpy.types.Scene.my_blend_mode = bpy.props.EnumProperty(
    name="Blend Mode",
    description="Choose a Blend Mode for material with Opacity maps",
    items=blend_modes,
    default='HASHED'
)

# Vocabulary with Displacement
disp_modes=[
    ('BUMP',"Bump Only","Bump Only"),
    ('DISPLACEMENT',"Displacement Only","Displacement Only"),
    ('BOTH',"Displacement and Bump","Displacement and Bump")
]
bpy.types.Scene.my_disp_mode=bpy.props.EnumProperty(
    name="Displacement Mode",
    description="Choose a Surface Displacement Mode for materials with Displacement maps",
    items=disp_modes,
    default='BOTH'
)

# _channel for each texture map
texture_keys=["basecolor","normal","roughness","metallic","opacity",
              "subsurface","specular","specular_tint",
              "anisotropic","sheen","sheen_tint","clearcoat","clearcoat_roughness",
              "ior","transmission","displacement","emission","ao"]

for key in texture_keys:
    setattr(bpy.types.Scene,key+"_channel",bpy.props.EnumProperty(
        name=key.capitalize()+" ",
        description=f"Select the output color channel for {key} map",
        items=[
            ('RGB','Channel','RGB Channel'),
            ('R','->RED','Red Channel'),
            ('G','->GREEN','Green Channel'),
            ('B','->BLUE','Blue Channel'),
        ],
        default='RGB',
    ))



def resolve_path(path):
    # Converting a Blender-specific relative path to an absolute path
    if path.startswith("//"):
        return bpy.path.abspath(path)
    return path

def update_material_list(self,context):
    bpy.ops.material.check_all()



def update_material_list(self, context):
    if not context or not context.scene:
        print("[BAM] Context or scene is not available.")
        return
    path = resolve_path(context.scene.texture_folder_path)
    if not os.path.exists(path):
        print(f"[update_material_list] Path not found: {path}")
        return []

    materials = detect_materials_in_folder(context.scene)
    material_list = context.scene.material_list
    material_list.clear()
    for material in materials:
        item = material_list.add()
        item.name = material
        item.checked = material not in bpy.data.materials


def with_temporary_cycles_renderer(func):
    def wrapper(*args, **kwargs):
        context = bpy.context
        if not context or not context.scene:
            print("Context or scene is not available.")
            return None

        addon_prefs = None
        try:
            addon_prefs = context.preferences.addons[__package__].preferences
        except KeyError:
            print("[BAM] Addon preferences not found. Using default settings.")

        original_renderer = None
        if addon_prefs and getattr(addon_prefs, 'switch_to_cycles', False):
            original_renderer = context.window.scene.render.engine
            print(f"[BAM] Original renderer: {original_renderer}")
            context.window.scene.render.engine = 'CYCLES'
            print("[BAM] Switched to Cycles")

        try:
            result = func(*args, **kwargs)
        finally:
            if addon_prefs and getattr(addon_prefs, 'switch_to_cycles', False) and original_renderer:
                print(f"[BAM] Switching back to {original_renderer} is Disabled")
                #context.window.scene.render.engine = original_renderer

        return result

    return wrapper


def detect_materials_in_folder(scene):
    path = resolve_path(scene.texture_folder_path)
    if not os.path.exists(path):
        print(f"[BAM] [detect_materials_in_folder] Path not found: {path}")
        return []
    
    materials = set()
    include_subfolders = scene.include_subfolders

    def check_suffixes(file, suffixes, format):
        file_lower = file.lower()
        for suffix in suffixes:
            if file_lower.endswith(f"{suffix.strip().lower()}.{format.lower()}"):
                return True
        return False

    def get_material_name(file, suffixes):
        for suffix in suffixes:
            suffix_with_format = suffix.strip().lower()
            if file.lower().endswith(suffix_with_format):
                return file[: -len(suffix_with_format)]
        return file

    if include_subfolders:
        for root, dirs, files in os.walk(path):
            for file in files:
                for format in image_formats.keys():
                    suffixes = getattr(scene, 'basecolor_suffix').split(",")
                    if check_suffixes(file, suffixes, format):
                        material_name = get_material_name(file, [f"{suffix.strip().lower()}.{format.lower()}" for suffix in suffixes])
                        materials.add(material_name)
    else:
        for item in os.listdir(path):
            for format in image_formats.keys():
                suffixes = getattr(scene, 'basecolor_suffix').split(",")
                if check_suffixes(item, suffixes, format):
                    material_name = get_material_name(item, [f"{suffix.strip().lower()}.{format.lower()}" for suffix in suffixes])
                    materials.add(material_name)

    return list(materials)



class MaterialNodeManager:
    def __init__(self, material):
        self.material = material
        self.node_tree = material.node_tree

        self.texture_coordinate_node = self.create_texture_coordinate_node()
        self.mapping_node = self.create_mapping_node()

    def create_texture_coordinate_node(self):
        # Check if Texture Coordinate node exist
        for node in self.node_tree.nodes:
            if node.type == 'TEX_COORD':
                return node  # use existing
        texture_coordinate_node = self.node_tree.nodes.new(type='ShaderNodeTexCoord')
        texture_coordinate_node.location = (-1400, 0)
        return texture_coordinate_node

    def create_mapping_node(self):
        # check if Mapping exist
        for node in self.node_tree.nodes:
            if node.type == 'MAPPING':
                return node 
        mapping_node = self.node_tree.nodes.new(type='ShaderNodeMapping')
        mapping_node.location = (-1200, 0)
        return mapping_node

    def connect_texture_coordinates(self,texture_node):
        # connect Texture Coordinate with textures
        self.node_tree.links.new(
            self.mapping_node.outputs['Vector'], 
            texture_node.inputs['Vector']
        )

    def connect_mapping(self,texture_node):
        # connect Mapping and texture
        self.node_tree.links.new(
            self.texture_coordinate_node.outputs['UV'],
            self.mapping_node.inputs['Vector']
        )


def ensure_principled_bsdf_connection(node_tree, shader, output):
    # Get the Surface input from the Material Output node
    output_surface_input = output.inputs['Surface']
    # Check if the Principled BSDF node is connected to this input
    if not output_surface_input.is_linked or (output_surface_input.links[0].from_node != shader):
        # Remove all existing connections to this input
        while output_surface_input.is_linked:
            node_tree.links.remove(output_surface_input.links[0])
        # Create a new connection from Principled BSDF to Material Output
        node_tree.links.new(output_surface_input, shader.outputs['BSDF'])



@with_temporary_cycles_renderer
def create_material_from_textures(path, scene):
    path = bpy.path.abspath(path)
    if not os.path.exists(path):
        print(f"[BAM] [create_material_from_textures(1)] Path not found: {path}")
        return []

    path = resolve_path(path)
    if not os.path.exists(path):
        print(f"[BAM] [create_material_from_textures(3)] Path not found: {path}")
        return []

    materials = []
    materials_textures = {}

    include_subfolders = scene.include_subfolders

    def process_texture_file(full_item_path, item_name, materials_textures):
        for info_key, info_value in textures_info.items():
            suffixes = getattr(scene, info_key + "_suffix").split(",")
            channel = getattr(scene, info_key + "_channel")
            for suffix in suffixes:
                suffix = suffix.strip().lower()
                for format in image_formats.keys():
                    texture_suffix_with_extension = suffix + "." + format.lower()
                    if item_name.lower().endswith(texture_suffix_with_extension):
                        material_name = item_name[: -len(texture_suffix_with_extension)]
                        if material_name not in materials_textures:
                            materials_textures[material_name] = []
                        materials_textures[material_name].append((info_key, channel, full_item_path))

    if include_subfolders:
        for root, dirs, files in os.walk(path):
            for item in files:
                full_item_path = os.path.join(root, item)
                process_texture_file(full_item_path, item, materials_textures)
    else:
        for item in os.listdir(path):
            full_item_path = os.path.join(path, item)
            process_texture_file(full_item_path, item, materials_textures)

    for material_name, textures in materials_textures.items():
        if not any(material.checked for material in scene.material_list if material.name == material_name):
            continue

        mat = bpy.data.materials.get(material_name)

        if not mat:
            mat = bpy.data.materials.new(name=material_name)

        mat.use_nodes = True
        nodes = mat.node_tree.nodes
        node_tree = mat.node_tree

        shader = None
        output = None
        rgb_curves = None

        for node in nodes:
            if node.type == "BSDF_PRINCIPLED":
                shader = node
            elif node.type == "OUTPUT_MATERIAL":
                output = node
            elif node.type == "ShaderNodeRGBCurve":
                rgb_curves = node

        if not shader:
            shader = nodes.new(type='ShaderNodeBsdfPrincipled')
            shader.location = (0, 0)

        if not output:
            output = nodes.new(type='ShaderNodeOutputMaterial')
            output.location = (400, 0)

        output_surface_input = output.inputs["Surface"]
        if not output_surface_input.is_linked:
            mat.node_tree.links.new(output_surface_input, shader.outputs["BSDF"])

        for node in nodes:
            if node != shader and node != output and node != rgb_curves:
                nodes.remove(node)

        ensure_principled_bsdf_connection(node_tree, shader, output)

        shader.inputs["Base Color"].default_value = (1.0, 1.0, 1.0, 1.0)

        if blender_version[0] >= 4:
            shader.inputs["Subsurface Weight"].default_value = 0.0
            shader.inputs["Specular IOR Level"].default_value = 0.5
            shader.inputs["Specular Tint"].default_value = (0.0, 0.0, 0.0, 1.0)
            shader.inputs["Transmission Weight"].default_value = 0.0
            shader.inputs["Sheen Weight"].default_value = 0.0
            shader.inputs["Sheen Tint"].default_value = (1.0, 1.0, 1.0, 1.0)
            shader.inputs["Emission Color"].default_value = (0.0, 0.0, 0.0, 1.0)
            shader.inputs["Coat Weight"].default_value = 0.0
            shader.inputs["Coat Roughness"].default_value = 0.03
        else:
            shader.inputs["Subsurface"].default_value = 0.0
            shader.inputs["Subsurface Color"].default_value = (1.0, 1.0, 1.0, 1.0)
            shader.inputs["Specular"].default_value = 0.5
            shader.inputs["Specular Tint"].default_value = 0.0
            shader.inputs["Transmission"].default_value = 0.0
            shader.inputs["Transmission Roughness"].default_value = 0.0
            shader.inputs["Sheen"].default_value = 0.0
            shader.inputs["Sheen Tint"].default_value = 0.5
            shader.inputs["Emission"].default_value = (0.0, 0.0, 0.0, 1.0)
            shader.inputs["Clearcoat"].default_value = 0.0
            shader.inputs["Clearcoat Roughness"].default_value = 0.03

        shader.inputs["Subsurface Radius"].default_value = (1.0, 1.0, 1.0)
        shader.inputs["Metallic"].default_value = 0.0
        shader.inputs["Roughness"].default_value = 0.5
        shader.inputs["Anisotropic"].default_value = 0.0
        shader.inputs["Anisotropic Rotation"].default_value = 0.0
        shader.inputs["IOR"].default_value = 1.45
        shader.inputs["Alpha"].default_value = 1.0

        vertical_location = 300

        # Create a dictionary to track which textures have been replaced
        replaced_textures = {key: False for key in textures_info.keys()}

        for info_key, channel, full_item_path in textures:
            if not getattr(scene, f"{info_key}_use_value"):
                full_item_path = resolve_path(full_item_path)
                if os.path.exists(full_item_path):
                    texture_node = nodes.new(type="ShaderNodeTexImage")
                    image_name = os.path.basename(full_item_path)

                    if image_name in bpy.data.images:
                        image = bpy.data.images[image_name]
                        if image.filepath_from_user() != full_item_path:
                            image.filepath = full_item_path
                        texture_node.image = image
                    else:
                        image = bpy.data.images.load(filepath=full_item_path)
                        texture_node.image = image

                    texture_node.location = (-800, vertical_location)
                    vertical_location -= 300

                    mix_node = mat.node_tree.nodes.get("MixNodeName")

                    if mix_node is None:
                        mix_node = mat.node_tree.nodes.new("ShaderNodeMixRGB")
                        mix_node.name = "MixNodeName"
                        mix_node.blend_type = 'MULTIPLY'
                        mix_node.inputs["Fac"].default_value = 1
                        mix_node.inputs["Color1"].default_value = (1, 1, 1, 1)
                        mix_node.inputs["Color2"].default_value = (1, 1, 1, 1)
                        mix_node.location = (-500, vertical_location)
                        vertical_location -= 0

                    if channel != 'RGB':
                        if info_key == "normal":
                            nrm_node = nodes.new(type="ShaderNodeNormalMap")
                            nrm_node.location = (-500, vertical_location)
                            vertical_location -= 5

                            separate_node = nodes.new(type="ShaderNodeSeparateRGB")
                            separate_node.location = (-200, vertical_location)
                            vertical_location -= 5

                            if scene.invert_normal_map_green_channel:
                                rgb_curves = nodes.new(type="ShaderNodeRGBCurve")
                                rgb_curves.location = (-500, vertical_location)

                                green_curve = rgb_curves.mapping.curves[1]

                                green_curve.points[0].location = (0.0, 1.0)
                                green_curve.points[1].location = (1.0, 0.0)

                                mat.node_tree.links.new(rgb_curves.inputs["Color"], texture_node.outputs["Color"])
                                mat.node_tree.links.new(nrm_node.inputs["Color"], rgb_curves.outputs["Color"])
                                mat.node_tree.links.new(separate_node.inputs["Image"], nrm_node.outputs["Normal"])
                            else:
                                mat.node_tree.links.new(nrm_node.inputs["Color"], texture_node.outputs["Color"])
                                mat.node_tree.links.new(separate_node.inputs["Image"], nrm_node.outputs["Normal"])
                        else:
                            separate_node = nodes.new(type="ShaderNodeSeparateRGB")
                            separate_node.location = (-200, vertical_location)
                            vertical_location -= 5
                            mat.node_tree.links.new(separate_node.inputs["Image"], texture_node.outputs["Color"])

                        if info_key == "normal":
                            mat.node_tree.links.new(shader.inputs["Normal"], separate_node.outputs[channel])
                            texture_node.image.colorspace_settings.name = scene.normal_colorspace
                        else:
                            if info_key == "basecolor":
                                mix_node.location = (texture_node.location.x + 10, texture_node.location.y)
                                mat.node_tree.links.new(mix_node.inputs[1], separate_node.outputs[channel])
                                mat.node_tree.links.new(shader.inputs["Base Color"], mix_node.outputs[0])
                                texture_node.image.colorspace_settings.name = scene.basecolor_colorspace

                                mat.node_tree.nodes.active = texture_node
                            elif info_key == "ao":
                                mat.node_tree.links.new(mix_node.inputs[2], separate_node.outputs[channel])
                                texture_node.image.colorspace_settings.name = scene.ao_colorspace
                            elif info_key == "roughness":
                                mat.node_tree.links.new(shader.inputs["Roughness"], separate_node.outputs[channel])
                                texture_node.image.colorspace_settings.name = scene.roughness_colorspace
                            elif info_key == "metallic":
                                mat.node_tree.links.new(shader.inputs["Metallic"], separate_node.outputs[channel])
                                texture_node.image.colorspace_settings.name = scene.metallic_colorspace
                            elif info_key == "opacity":
                                mat.node_tree.links.new(shader.inputs["Alpha"], separate_node.outputs[channel])
                                texture_node.image.colorspace_settings.name = scene.opacity_colorspace
                            elif info_key == "subsurface":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Subsurface Weight"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.subsurface_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Subsurface"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.subsurface_colorspace
                            elif info_key == "specular":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Specular IOR Level"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.specular_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Specular"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.specular_colorspace
                            
                                
                            elif info_key == "specular_tint":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Specular Tint"], separate_node.outputs[channel])
                                else:
                                    mat.node_tree.links.new(shader.inputs["Specular Tint"], separate_node.outputs[channel].outputs['R'])
                                texture_node.image.colorspace_settings.name = scene.specular_tint_colorspace
                            
                                
                            elif info_key == "anisotropic":
                                mat.node_tree.links.new(shader.inputs["Anisotropic"], separate_node.outputs[channel])
                                texture_node.image.colorspace_settings.name = scene.anisotropic_colorspace
                            elif info_key == "sheen":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Sheen Weight"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.sheen_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Sheen"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.sheen_colorspace
                            elif info_key == "sheen_tint":
                                mat.node_tree.links.new(shader.inputs["Sheen Tint"], separate_node.outputs[channel])
                                texture_node.image.colorspace_settings.name = scene.sheen_tint_colorspace
                            elif info_key == "clearcoat":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Coat Weight"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.clearcoat_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Clearcoat"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.clearcoat_colorspace
                            elif info_key == "clearcoat_roughness":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Coat Roughness"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.clearcoat_roughness_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Clearcoat Roughness"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.clearcoat_roughness_colorspace
                            elif info_key == "ior":
                                mat.node_tree.links.new(shader.inputs["IOR"], separate_node.outputs[channel])
                                texture_node.image.colorspace_settings.name = scene.ior_colorspace
                            elif info_key == "transmission":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Transmission Weight"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.transmission_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Transmission"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.transmission_colorspace
                            elif info_key == "displacement":
                                disp_node = nodes.new(type="ShaderNodeDisplacement")
                                disp_node.inputs["Scale"].default_value = 0.1
                                disp_node.location = (-500, vertical_location)
                                vertical_location -= 5

                                mat.node_tree.links.new(disp_node.inputs["Height"], separate_node.outputs[channel])
                                mat.node_tree.links.new(output.inputs["Displacement"], disp_node.outputs["Displacement"])
                                texture_node.image.colorspace_settings.name = scene.displacement_colorspace
                            elif info_key == "emission":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Emission Color"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.emission_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Emission"], separate_node.outputs[channel])
                                    texture_node.image.colorspace_settings.name = scene.emission_colorspace

                    else:
                        if isinstance(texture_node, bpy.types.ShaderNodeTexImage):
                            if info_key == "normal":
                                normal_map_node = nodes.new(type="ShaderNodeNormalMap")
                                normal_map_node.location = (-200, vertical_location)
                                vertical_location -= 50

                                if scene.invert_normal_map_green_channel:
                                    rgb_curves = nodes.new(type="ShaderNodeRGBCurve")
                                    rgb_curves.location = (-500, vertical_location)

                                    green_curve = rgb_curves.mapping.curves[1]

                                    green_curve.points[0].location = (0.0, 1.0)
                                    green_curve.points[1].location = (1.0, 0.0)

                                    mat.node_tree.links.new(rgb_curves.inputs["Color"], texture_node.outputs["Color"])
                                    mat.node_tree.links.new(normal_map_node.inputs["Color"], rgb_curves.outputs["Color"])
                                else:
                                    mat.node_tree.links.new(normal_map_node.inputs["Color"], texture_node.outputs["Color"])

                                mat.node_tree.links.new(shader.inputs["Normal"], normal_map_node.outputs["Normal"])
                                texture_node.image.colorspace_settings.name = scene.normal_colorspace
                            elif info_key == "ao":
                                mat.node_tree.links.new(mix_node.inputs[2], texture_node.outputs["Color"])
                                texture_node.image.colorspace_settings.name = scene.ao_colorspace
                            elif info_key == "basecolor":
                                mat.node_tree.links.new(mix_node.inputs[1], texture_node.outputs["Color"])
                                mat.node_tree.links.new(shader.inputs["Base Color"], mix_node.outputs[0])
                                texture_node.image.colorspace_settings.name = scene.basecolor_colorspace

                                mat.node_tree.nodes.active = texture_node
                            elif info_key == "roughness":
                                mat.node_tree.links.new(shader.inputs["Roughness"], texture_node.outputs["Color"])
                                texture_node.image.colorspace_settings.name = scene.roughness_colorspace
                            elif info_key == "metallic":
                                mat.node_tree.links.new(shader.inputs["Metallic"], texture_node.outputs["Color"])
                                texture_node.image.colorspace_settings.name = scene.metallic_colorspace
                            elif info_key == "opacity":
                                mat.node_tree.links.new(shader.inputs["Alpha"], texture_node.outputs["Color"])
                                texture_node.image.colorspace_settings.name = scene.opacity_colorspace
                            elif info_key == "subsurface":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Subsurface Weight"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.subsurface_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Subsurface"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.subsurface_colorspace
                            elif info_key == "specular":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Specular IOR Level"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.specular_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Specular"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.specular_colorspace
                            elif info_key == "specular_tint":
                                mat.node_tree.links.new(shader.inputs["Specular Tint"], texture_node.outputs["Color"])
                                texture_node.image.colorspace_settings.name = scene.specular_tint_colorspace
                            elif info_key == "anisotropic":
                                mat.node_tree.links.new(shader.inputs["Anisotropic"], texture_node.outputs["Color"])
                                texture_node.image.colorspace_settings.name = scene.anisotropic_colorspace
                            elif info_key == "sheen":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Sheen Weight"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.sheen_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Sheen"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.sheen_colorspace
                            elif info_key == "sheen_tint":
                                mat.node_tree.links.new(shader.inputs["Sheen Tint"], texture_node.outputs["Color"])
                                texture_node.image.colorspace_settings.name = scene.sheen_tint_colorspace
                            elif info_key == "clearcoat":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Coat Weight"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.clearcoat_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Clearcoat"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.clearcoat_colorspace
                            elif info_key == "clearcoat_roughness":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Coat Roughness"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.clearcoat_roughness_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Clearcoat Roughness"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.clearcoat_roughness_colorspace
                            elif info_key == "ior":
                                mat.node_tree.links.new(shader.inputs["IOR"], texture_node.outputs["Color"])
                                texture_node.image.colorspace_settings.name = scene.ior_colorspace
                            elif info_key == "transmission":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Transmission Weight"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.transmission_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Transmission"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.transmission_colorspace
                            elif info_key == "displacement":
                                disp_node = nodes.new(type="ShaderNodeDisplacement")
                                disp_node.inputs["Scale"].default_value = 0.1
                                disp_node.location = (-500, vertical_location)
                                vertical_location -= 5

                                mat.node_tree.links.new(disp_node.inputs["Height"], texture_node.outputs["Color"])
                                mat.node_tree.links.new(output.inputs["Displacement"], disp_node.outputs["Displacement"])
                                texture_node.image.colorspace_settings.name = scene.displacement_colorspace
                            elif info_key == "emission":
                                if blender_version[0] >= 4:
                                    mat.node_tree.links.new(shader.inputs["Emission Color"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.emission_colorspace
                                else:
                                    mat.node_tree.links.new(shader.inputs["Emission"], texture_node.outputs["Color"])
                                    texture_node.image.colorspace_settings.name = scene.emission_colorspace

        for info_key in textures_info.keys():
            if getattr(scene, f"{info_key}_use_value") and not replaced_textures[info_key]:
                custom_value = getattr(scene, f"{info_key}_custom_value")
                if info_key == "basecolor":
                    shader.inputs["Base Color"].default_value = custom_value
                elif info_key == "roughness":
                    shader.inputs["Roughness"].default_value = custom_value[0]
                elif info_key == "metallic":
                    shader.inputs["Metallic"].default_value = custom_value[0]
                elif info_key == "opacity":
                    shader.inputs["Alpha"].default_value = custom_value[0]
                elif info_key == "subsurface":
                    if blender_version[0] >= 4:
                        shader.inputs["Subsurface Weight"].default_value = custom_value[0]
                    else:
                        shader.inputs["Subsurface"].default_value = custom_value[0]
                
                elif info_key == "specular":
                    if blender_version[0] >= 4:
                        shader.inputs["Specular IOR Level"].default_value = custom_value[0]
                    else:
                        shader.inputs["Specular"].default_value = custom_value[0]
                
                elif info_key == "specular_tint":
                    if blender_version[0] >= 4:
                        shader.inputs["Specular Tint"].default_value = custom_value
                    else:
                        shader.inputs["Specular Tint"].default_value = custom_value[0]
                    
                elif info_key == "anisotropic":
                    shader.inputs["Anisotropic"].default_value = custom_value[0]
                elif info_key == "sheen":
                    if blender_version[0] >= 4:
                        shader.inputs["Sheen Weight"].default_value = custom_value[0]
                    else:
                        shader.inputs["Sheen"].default_value = custom_value[0]
                elif info_key == "sheen_tint":
                    shader.inputs["Sheen Tint"].default_value = custom_value
                elif info_key == "clearcoat":
                    if blender_version[0] >= 4:
                        shader.inputs["Coat Weight"].default_value = custom_value[0]
                    else:
                        shader.inputs["Clearcoat"].default_value = custom_value[0]
                elif info_key == "clearcoat_roughness":
                    if blender_version[0] >= 4:
                        shader.inputs["Coat Roughness"].default_value = custom_value[0]
                    else:
                        shader.inputs["Clearcoat Roughness"].default_value = custom_value[0]
                elif info_key == "ior":
                    shader.inputs["IOR"].default_value = custom_value[0]
                elif info_key == "transmission":
                    if blender_version[0] >= 4:
                        shader.inputs["Transmission Weight"].default_value = custom_value[0]
                    else:
                        shader.inputs["Transmission"].default_value = custom_value[0]
                elif info_key == "emission":
                    if blender_version[0] >= 4:
                        shader.inputs["Emission Color"].default_value = custom_value
                    else:
                        shader.inputs["Emission"].default_value = custom_value

        materials.append(mat)

        for material in materials:
            material_node_manager = MaterialNodeManager(material)
            texture_nodes = [node for node in material.node_tree.nodes if node.type == 'TEX_IMAGE']

            for texture_node in texture_nodes:
                material_node_manager.connect_texture_coordinates(texture_node)
                material_node_manager.connect_mapping(texture_node)

        
        #Special event for shaders with alpha maps
        texture_found = False
        print("Starting blend mode check for", material_name)
        print("Opacity suffixes:", getattr(scene, "opacity_suffix"))

        for format in image_formats.keys():
            suffixes = getattr(scene, "opacity_suffix").split(",")
            #print("Checking format:", format)
            for suffix in suffixes:
                texture_suffix_with_extension = suffix.strip() + "." + format.lower()
                if include_subfolders:
                    for root, _, files in os.walk(path):
                        for file in files:
                            texture_path = os.path.join(root, file)
                            if texture_path.endswith(texture_suffix_with_extension) and material_name in texture_path:
                                #print("Checking path:", texture_path)
                                if os.path.exists(texture_path):
                                    #print(f"Found texture at {texture_path}")
                                    print(f"Setting blend method for {material_name} using {texture_path}")
                                    mat.blend_method = scene.my_blend_mode
                                    texture_found = True
                                    break
                        if texture_found:
                            break
                else:
                    texture_path = os.path.join(path, material_name + texture_suffix_with_extension)
                    #print("Checking path:", texture_path)
                    if os.path.exists(texture_path):
                        #print(f"Found texture at {texture_path}")
                        print(f"Setting blend method for {material_name} using {texture_path}")
                        mat.blend_method = scene.my_blend_mode
                        texture_found = True
                        break
            if texture_found:
                break

        if not texture_found:
            print(f"No texture found for material {material_name} with specified suffixes and formats")
        else:
            print(f"Blend method set to {mat.blend_method} for material {material_name}")
            
            
            
        #Special even for shaders with displacement maps
        texture_found2 = False
        print("Starting Disp mode check for", material_name)
        print("Displcement suffixes:", getattr(scene, "displacement_suffix"))

        for format in image_formats.keys():
            suffixes = getattr(scene, "displacement_suffix").split(",")
            print("Checking format:", format)
            for suffix in suffixes:
                texture_suffix_with_extension = suffix.strip() + "." + format.lower()
                if include_subfolders:
                    for root, _, files in os.walk(path):
                        for file in files:
                            texture_path = os.path.join(root, file)
                            if texture_path.endswith(texture_suffix_with_extension) and material_name in texture_path:
                                #print("Checking path:", texture_path)
                                if os.path.exists(texture_path):
                                    #print(f"Found texture at {texture_path}")
                                    print(f"Setting Disp mode for {material_name} using {texture_path}")
                                    
                                    if blender_version[0] >= 4 and blender_version[1] >= 1:
                                        mat.displacement_method = scene.my_disp_mode
                                    else:
                                        mat.cycles.displacement_method = scene.my_disp_mode
                                    
                                    
                                    texture_found2 = True
                                    
                                    break
                        if texture_found2:
                            break
                else:
                    texture_path = os.path.join(path, material_name + texture_suffix_with_extension)
                    #print("Checking path:", texture_path)
                    if os.path.exists(texture_path):
                        #print(f"Found texture at {texture_path}")
                        print(f"Setting Disp mode for {material_name} using {texture_path}")
                        
                        
                        if blender_version[0] >= 4 and blender_version[1] >= 1:
                            mat.displacement_method = scene.my_disp_mode
                        else:
                            mat.cycles.displacement_method = scene.my_disp_mode
                        
                        
                        texture_found2 = True
                        break
            if texture_found2:
                break

        if not texture_found2:
            print(f"No texture found for material {material_name} with specified suffixes and formats")
        else:
            if blender_version[0] >= 4 and blender_version[1] >= 1:
                print(f"Disp mode set to {mat.displacement_method} for material {material_name}")
            else:
                print(f"Disp mode set to {mat.cycles.displacement_method} for material {material_name}")
            
                    

    return materials

   



# Operators to set all checkboxes to True or False
class MATERIAL_OT_check_all(bpy.types.Operator):
    bl_idname="material.check_all"
    bl_label="Check All"
    bl_description="Check all materials"

    def execute(self,context):
        for material in context.scene.material_list:
            material.checked=True
        return {'FINISHED'}


class MATERIAL_OT_uncheck_all(bpy.types.Operator):
    bl_idname="material.uncheck_all"
    bl_label="Uncheck All"
    bl_description="Uncheck all materials"

    def execute(self,context):
        for material in context.scene.material_list:
            material.checked=False
        return {'FINISHED'}

class MATERIAL_OT_invert_selection(bpy.types.Operator):
    bl_idname = "material.invert_selection"
    bl_label = "Invert Selection"
    bl_description = "Invert the selection of materials"

    def execute(self, context):
        for material in context.scene.material_list:
            material.checked = not material.checked
        return {'FINISHED'}

# colorspace func
def match_textures_colorspace(scene):
    for image in bpy.data.images:
        for info_key,_ in textures_info.items():
            suffix=getattr(scene,info_key+"_suffix")
            for format in image_formats.keys():
                texture_suffix_with_extension=suffix+"."+format.lower()
                if image.name.endswith(texture_suffix_with_extension):
                    colorspace=getattr(scene,info_key+"_colorspace")
                    image.colorspace_settings.name=colorspace
                    break


class MATERIAL_OT_match_colorspaces(bpy.types.Operator):
    bl_idname="material.match_colorspaces"
    bl_label="Scene ColorSpace Match"
    bl_description="Apply Color Space rules to all textures in Project."

    def execute(self,context):
        match_textures_colorspace(context.scene)
        self.report({'INFO'},"Color spaces matched!")
        return {'FINISHED'}


        
# Main button for creating and updating materials
class MATERIAL_OT_create_bsdf(bpy.types.Operator):
    bl_idname = "material.create_bsdf"
    bl_label = "Create and update Materials"
    bl_description = "Batch Materials Creator and Updater"

    def execute(self, context):
        path = resolve_path(context.scene.texture_folder_path)
        if not os.path.exists(path):
            self.report({'ERROR'}, f"[MATERIAL_OT_create_bsdf] Path not found: {path}")
            return {'CANCELLED'}

        create_material_from_textures(path, context.scene)
        return {'FINISHED'}



class MaterialItem(bpy.types.PropertyGroup):
    name: bpy.props.StringProperty()
    checked: bpy.props.BoolProperty(default=True)
    temp_checked: bpy.props.BoolProperty(default=True)  # Temporary field to store state

    def is_material_exists(self):
        return self.name in bpy.data.materials

    @classmethod
    def register(cls):
        bpy.types.PropertyGroup.checked = bpy.props.BoolProperty(
            name="Checked",
            get=cls.get_checked,
            set=cls.set_checked
        )

    def get_checked(self):
        return self["checked"] if "checked" in self else True

    def set_checked(self, value):
        self["checked"] = value
        self.temp_checked = value  # Update temporary state as well


class MATERIAL_OT_dummy_operator(bpy.types.Operator):
    bl_idname="material.dummy_operator"
    bl_label="Hi Stranger"
    bl_description="Material already exists in scene"

    def execute(self,context):
        return {'PASS_THROUGH'}


class MATERIAL_UL_material_list(bpy.types.UIList):
    
    def draw_item(self,context,layout,data,item,icon,active_data,active_propname,index):
        if self.layout_type in {'DEFAULT','COMPACT'}:
            split=layout.split(factor=0.05)

            if item.checked:
                icon_check='CHECKBOX_HLT'
            else:
                icon_check='CHECKBOX_DEHLT'

            split.prop(item,"checked",text="",toggle=True,icon=icon_check)

            # Check if the material exists in the scene
            if item.name in bpy.data.materials:
                icon_text='SCENE_DATA'
            else:
                icon_text='NONE'

            col=split.column(align=True)
            sub=col.row(align=True)
            sub.label(text=f"{index+1}. {item.name}")

            # If we have an icon to display, add it
            if icon_text!='NONE':
                sub.operator("material.dummy_operator",text="",emboss=False,icon=icon_text)


        elif self.layout_type in {'GRID'}:
            layout.prop(item,"checked",text="",toggle=True)

    def draw(self,context):
        layout=self.layout
        scene=context.scene

        for material in scene.material_list:
            material_row=layout.row(align=True)
            material_row.template_list("MATERIAL_UL_material_list","",scene,"material_list",scene,
                                       "material_list_index",rows=12)




def init_props():
    bpy.types.Scene.invert_normal_map_green_channel = bpy.props.BoolProperty(
        name="Invert Normal Map Green Channel",
        description="Invert the green channel of the normal map",
        default=False
    )

    for key in texture_keys:
        if key == "specular_tint" and blender_version[0] < 4:
            setattr(bpy.types.Scene, f"{key}_use_value", bpy.props.BoolProperty(
                name="Value",
                description=f"Use custom value for {key} map",
                default=False
            ))
            setattr(bpy.types.Scene, f"{key}_custom_value", bpy.props.FloatProperty(
                name=f"{key.capitalize()} Custom Value",
                description=f"Custom value for {key} map",
                default=0.5,
                min=0.0,
                max=1.0
            ))
        else:
            setattr(bpy.types.Scene, f"{key}_use_value", bpy.props.BoolProperty(
                name="Value",
                description=f"Use custom value for {key} map",
                default=False
            ))
            setattr(bpy.types.Scene, f"{key}_custom_value", bpy.props.FloatVectorProperty(
                name=f"{key.capitalize()} Custom Value",
                description=f"Custom value for {key} map",
                subtype='COLOR_GAMMA' if key in ["basecolor", "emission", "specular_tint"] else 'NONE',
                size=4 if key in ["basecolor", "emission", "specular_tint"] else 1,
                default=(1.0, 1.0, 1.0, 1.0) if key in ["basecolor", "emission", "specular_tint"] else (1.0,),
                min=0.0,
                max=1.0
            ))

def clear_props():
    del bpy.types.Scene.invert_normal_map_green_channel
    
    for key in texture_keys:
        del bpy.types.Scene[f"{key}_use_value"]
        del bpy.types.Scene[f"{key}_custom_value"]
        
        
def update_materials_count(scene):
    path = resolve_path(scene.texture_folder_path)
    if not os.path.exists(path):
        print(f"[update_materials_count] Path not found: {path}")
        return 0

    materials_count = 0

    def check_suffixes(file, suffixes, format):
        file_lower = file.lower()
        for suffix in suffixes:
            if file_lower.endswith(f"{suffix.strip().lower()}.{format.lower()}"):
                return True
        return False

    if scene.include_subfolders:
        for root, dirs, files in os.walk(path):
            for file in files:
                for format in image_formats.keys():
                    suffixes = getattr(scene, 'basecolor_suffix').split(",")
                    if check_suffixes(file, suffixes, format):
                        materials_count += 1
                        break
    else:
        for item in os.listdir(path):
            for format in image_formats.keys():
                suffixes = getattr(scene, 'basecolor_suffix').split(",")
                if check_suffixes(item, suffixes, format):
                    materials_count += 1
                    break

    return materials_count

class MATERIAL_PT_panel(bpy.types.Panel):
    bl_label = "[BAM] AutoMat"
    bl_idname = "MATERIAL_PT_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = '[BAM] AutoMat'

    def draw(self, context):
        layout = self.layout
        scene = context.scene

        row = layout.row(align=True)
        split = row.split(factor=0.4)
        split.label(text="TextureFolder:")
        split.prop(scene, "texture_folder_path", text="")
        row.prop(scene, "include_subfolders", text="Subfolders")

        row = layout.row(align=True)

        if scene.texture_folder_path:
            box = layout.box()
            row = box.row()
            row.scale_y = 1
            row.alignment = 'CENTER'

            materials_count = update_materials_count(scene)
            row.label(text=f"Detected Materials in Folder: ", icon="MATSHADERBALL")
            row.label(text=f"{materials_count}", icon="KEYTYPE_JITTER_VEC" if materials_count > 0 else "KEYTYPE_EXTREME_VEC")

            row = box.row(align=True)

            row.alert = True
            row.operator("material.create_bsdf", icon="FORCE_TEXTURE")
            row = box.row(align=True)
            row.operator("material.match_colorspaces", icon="RESTRICT_COLOR_ON")

            box = layout.box()
            row = box.row()
            row.label(text=f"Definitions Presets", icon="PRESET")
            row = box.row(align=True)
            row.prop(scene, "preset_list", text="Presets")
            row = box.row()
            row.operator("material.save_preset_dialog", text="Save Preset")
            row.operator("material.delete_preset", text="Delete Preset")

            box = layout.box()
            row = box.row()

            row.prop(scene, "show_texture_settings", icon='TRIA_DOWN' if scene.show_texture_settings else 'TRIA_RIGHT', emboss=False, text="TextureMap Definitions:")
            if scene.show_texture_settings:
                row = box.row(align=True)
                row.prop(scene, "invert_normal_map_green_channel")
                row = box.row(align=True)
                split = row.split(factor=0.5)
                split.label(text="Blend Mode for Opacity")
                split.prop(scene, "my_blend_mode", text="")

                row = box.row(align=True)
                split = row.split(factor=0.5)
                split.label(text="Displacement Mode")
                split.prop(scene, "my_disp_mode", text="")

                main_suffixes = ["basecolor", "normal", "roughness", "metallic", "ao", "opacity", "displacement", "emission"]
                for suffix in main_suffixes:
                    display_name = textures_info[suffix]["display_name"]

                    row = box.row(align=True)
                    split = row.split(factor=0.3)

                    if scene.get(f"{suffix}_use_value", False):
                        row = box.row(align=True)
                        row.prop(scene, f"{suffix}_custom_value", text=display_name)
                    else:
                        split.label(text=display_name)
                        split.prop(scene, f"{suffix}_suffix", text="")
                        split.prop(scene, f"{suffix}_channel", text="")
                        row.prop(scene, f"{suffix}_colorspace", text="")
                    row.prop(scene, f"{suffix}_use_value", text="")

                row = box.row()
                row.prop(scene, "show_advanced_texture_settings", icon='TRIA_DOWN' if scene.show_advanced_texture_settings else 'TRIA_RIGHT', emboss=True, text="Additional TextureMap Definitions:")

                if scene.show_advanced_texture_settings:
                    advanced_suffixes = ["subsurface", "specular", "specular_tint", "anisotropic", "sheen", "sheen_tint", "clearcoat", "clearcoat_roughness", "ior", "transmission"]
                    for suffix in advanced_suffixes:
                        display_name = textures_info[suffix]["display_name"]

                        row = box.row(align=True)
                        split = row.split(factor=0.3)

                        if scene.get(f"{suffix}_use_value", False):
                            row = box.row(align=True)
                            row.prop(scene, f"{suffix}_custom_value", text=display_name)
                        else:
                            split.label(text=display_name)
                            split.prop(scene, f"{suffix}_suffix", text="")
                            split.prop(scene, f"{suffix}_channel", text="")
                            row.prop(scene, f"{suffix}_colorspace", text="")
                        row.prop(scene, f"{suffix}_use_value", text="")

            box = layout.box()
            row = box.row()
            row.scale_y = 1
            row.alignment = 'LEFT'
            row.label(text="Materials Create/Update filter.", icon="FILTER")

            row = box.row(align=True)
            row.operator("material.check_all", icon='CHECKBOX_HLT')
            row.operator("material.uncheck_all", icon='CHECKBOX_DEHLT')
            row.operator("material.invert_selection", icon='ARROW_LEFTRIGHT')

            detected_materials = detect_materials_in_folder(scene)
            row = box.row(align=True)
            row = box.row(align=True)
            row.template_list("MATERIAL_UL_material_list", "", scene, "material_list", scene, "material_list_index")

            if scene.material_list:
                material = scene.material_list[scene.material_list_index]
        else:
            layout.label(text="Please select a texture folder", icon="ERROR")

        
# Operator to save the preset
class MATERIAL_OT_save_preset_dialog(bpy.types.Operator):
    bl_idname = "material.save_preset_dialog"
    bl_label = "Save Preset"
    bl_description = "Save Material Preset"
    
    preset_name: bpy.props.StringProperty(name="Preset Name")

    def invoke(self, context, event):
        wm = context.window_manager
        return wm.invoke_props_dialog(self)
    
    def execute(self, context):
        preset_file_path = os.path.join(addon_dir, "material_presets.json")
        presets = {}

        scene = context.scene
        preset_name = self.preset_name.strip()
        if not preset_name:
            self.report({'WARNING'}, "Preset name cannot be empty")
            return {'CANCELLED'}

        if os.path.exists(preset_file_path):
            with open(preset_file_path, 'r') as infile:
                presets = json.load(infile)

        settings = {
            "suffixes": {key: getattr(scene, f"{key}_suffix").split(",") for key in texture_keys},
            "channels": {key: getattr(scene, f"{key}_channel") for key in texture_keys},
            "colorspaces": {key: getattr(scene, f"{key}_colorspace") for key in texture_keys},
            "additional_settings": {
                "invert_normal_map_green_channel": scene.invert_normal_map_green_channel,
                "blend_mode": scene.my_blend_mode,
                "displacement_mode": scene.my_disp_mode
            }
        }

        presets[preset_name] = settings

        with open(preset_file_path, 'w') as outfile:
            json.dump(presets, outfile, indent=4)

        self.report({'INFO'}, "Preset saved")
        return {'FINISHED'}


# Operator to delete a preset
class MATERIAL_OT_delete_preset(bpy.types.Operator):
    bl_idname = "material.delete_preset"
    bl_label = "Delete Material Preset"

    def execute(self, context):
        preset_file_path = os.path.join(addon_dir, "material_presets.json")
        # Load existing presets
        if os.path.exists(preset_file_path):
            with open(preset_file_path, 'r') as infile:
                presets = json.load(infile)

            # Delete the selected preset
            preset_name = context.scene.preset_list
            if preset_name in presets:
                del presets[preset_name]

                with open(preset_file_path, 'w') as outfile:
                    json.dump(presets, outfile)

                self.report({'INFO'}, "Preset deleted")

        return {'FINISHED'}



def update_preset_items(scene, context):
    preset_file_path = os.path.join(addon_dir, "material_presets.json")
    items = []
    try:
        with open(preset_file_path, 'r') as infile:
            presets = json.load(infile)
            for name, _ in presets.items():
                items.append((name, name, ""))
    except FileNotFoundError:
        # If no file is found, return empty list or list with default value
        items = [('DEFAULT', 'Default', 'Default settings')]

    return items


def apply_preset_settings(scene, preset_name):
    preset_file_path = os.path.join(addon_dir, "material_presets.json")
    with open(preset_file_path, 'r') as infile:
        presets = json.load(infile)

    preset_settings = presets.get(preset_name, None)
    if preset_settings is None:
        print("Preset not found:", preset_name)
        return

    checked_states = {item.name: item.checked for item in scene.material_list}

    for key, value in preset_settings["suffixes"].items():
        setattr(scene, f"{key}_suffix", ",".join(value) if isinstance(value, list) else value)

    for key, value in preset_settings["channels"].items():
        setattr(scene, f"{key}_channel", value)

    for key, value in preset_settings["colorspaces"].items():
        setattr(scene, f"{key}_colorspace", value)

    additional_settings = preset_settings.get("additional_settings", {})
    scene.invert_normal_map_green_channel = additional_settings.get("invert_normal_map_green_channel", False)
    scene.my_blend_mode = additional_settings.get("blend_mode", "HASHED")
    scene.my_disp_mode = additional_settings.get("displacement_mode", "BOTH")

    for item in scene.material_list:
        item.checked = checked_states.get(item.name, True)

    for window in bpy.context.window_manager.windows:
        for area in window.screen.areas:
            if area.type in ['PROPERTIES', 'VIEW_3D']:
                area.tag_redraw()


def on_preset_change(self, context):
    scene = context.scene
    preset_name = scene.preset_list
    apply_preset_settings(scene, preset_name)

    # Update the UI
    for area in bpy.context.screen.areas:
        if area.type in ['PROPERTIES', 'VIEW_3D']:
            area.tag_redraw()


def load_handler(dummy):
    scene = bpy.context.scene

def update_path(scene, context):
    absolute_path = bpy.path.abspath(scene.texture_folder_path)
    if scene.texture_folder_path != absolute_path:
        scene.texture_folder_path = absolute_path
        context.area.tag_redraw()


def combined_update(self, context):
    update_path(self, context)
    update_material_list(self, context)


if blender_version[0] >= 4:
    texture_properties = {
        "basecolor": ("_basecolor", "sRGB", (1.0, 1.0, 1.0, 1.0)),
        "normal": ("_normal", "Non-Color", (1.0,)),
        "roughness": ("_roughness", "Non-Color", (1.0,)),
        "metallic": ("_metallic", "Non-Color", (1.0,)),
        "opacity": ("_opacity", "Non-Color", (1.0,)),
        "subsurface": ("_subsurface", "sRGB", (1.0,)),
        "ao": ("_ao", "sRGB", (1.0,)),
        "specular": ("_specular", "Non-Color", (1.0,)),
        "specular_tint": ("_specular_tint", "Non-Color", (1.0, 1.0, 1.0, 1.0)),
        "anisotropic": ("_anisotropic", "Non-Color", (1.0,)),
        "sheen": ("_sheen", "Non-Color", (1.0,)),
        "sheen_tint": ("_sheen_tint", "Non-Color", (1.0, 1.0, 1.0, 1.0)),
        "clearcoat": ("_clearcoat", "Non-Color", (1.0,)),
        "clearcoat_roughness": ("_clearcoat_roughness", "Non-Color", (1.0,)),
        "ior": ("_ior", "Non-Color", (1.0,)),
        "transmission": ("_transmission", "Non-Color", (1.0,)),
        "displacement": ("_height", "Non-Color", (1.0,)),
        "emission": ("_emissive", "sRGB", (1.0, 1.0, 1.0, 1.0))
    }
else:
    texture_properties = {
        "basecolor": ("_basecolor", "sRGB", (1.0, 1.0, 1.0, 1.0)),
        "normal": ("_normal", "Non-Color", (1.0,)),
        "roughness": ("_roughness", "Non-Color", (1.0,)),
        "metallic": ("_metallic", "Non-Color", (1.0,)),
        "opacity": ("_opacity", "Non-Color", (1.0,)),
        "subsurface": ("_subsurface", "sRGB", (1.0,)),
        "ao": ("_ao", "sRGB", (1.0,)),
        "specular": ("_specular", "Non-Color", (1.0,)),
        "specular_tint": ("_specular_tint", "Non-Color", (0.5,)),
        "anisotropic": ("_anisotropic", "Non-Color", (1.0,)),
        "sheen": ("_sheen", "Non-Color", (1.0,)),
        "sheen_tint": ("_sheen_tint", "Non-Color", (1.0,)),
        "clearcoat": ("_clearcoat", "Non-Color", (1.0,)),
        "clearcoat_roughness": ("_clearcoat_roughness", "Non-Color", (1.0,)),
        "ior": ("_ior", "Non-Color", (1.0,)),
        "transmission": ("_transmission", "Non-Color", (1.0,)),
        "displacement": ("_height", "Non-Color", (1.0,)),
        "emission": ("_emissive", "sRGB", (1.0, 1.0, 1.0, 1.0))
    }


def register():
    bpy.types.Scene.texture_folder_path = bpy.props.StringProperty(
        name="Texture Folder",
        subtype='DIR_PATH',
        update=combined_update,
        get=lambda self: bpy.path.abspath(self['texture_folder_path']) if 'texture_folder_path' in self else '',
        set=lambda self, value: self.__setitem__('texture_folder_path', bpy.path.abspath(value))
    )

    register_class(MaterialItem)
    bpy.types.Scene.material_list = bpy.props.CollectionProperty(type=MaterialItem)
    bpy.types.Scene.material_list_index = bpy.props.IntProperty()
    register_class(MATERIAL_OT_create_bsdf)
    register_class(MATERIAL_PT_panel)


    register_class(MATERIAL_OT_check_all)
    register_class(MATERIAL_OT_uncheck_all)
    register_class(MATERIAL_OT_invert_selection)
    register_class(MATERIAL_OT_dummy_operator)

    bpy.types.Scene.new_preset_name = bpy.props.StringProperty(name="New Preset Name")

    register_class(MATERIAL_OT_save_preset_dialog)
    register_class(MATERIAL_OT_delete_preset)

    bpy.app.handlers.load_post.append(load_handler)

    bpy.types.Scene.show_advanced_texture_settings = bpy.props.BoolProperty(
        name="Show Advanced Texture Settings",
        description="Toggle the visibility of advanced texture settings",
        default=False
    )

    bpy.types.Scene.show_texture_settings = bpy.props.BoolProperty(
        name="Show Texture Settings",
        description="Toggle the visibility of texture settings",
        default=True
    )

    bpy.types.Scene.invert_normal_map_green_channel = bpy.props.BoolProperty(
        name="Flip Normal Green Channel",
        description="Invert the Green channel of the normal map using RGB curves",
        default=False
    )



    for key, (suffix_default, colorspace_default, custom_value_default) in texture_properties.items():
        setattr(bpy.types.Scene, f"{key}_suffix", bpy.props.StringProperty(
            name=f"{key.capitalize()} Suffix",
            default=suffix_default,
            update=update_material_list
        ))
        setattr(bpy.types.Scene, f"{key}_colorspace", bpy.props.EnumProperty(
            items=colorspace_items,
            name=f"{key.capitalize()}: ColorSpace",
            default=colorspace_default
        ))
        setattr(bpy.types.Scene, f"{key}_use_value", bpy.props.BoolProperty(
            name=f"Value",
            description=f"Use custom value for {key.capitalize()} or set a texture",
            default=False
        ))
        setattr(bpy.types.Scene, f"{key}_custom_value", bpy.props.FloatVectorProperty(
            name=f"{key.capitalize()} Custom Value",
            description=f"Custom value for {key.capitalize()} when texture is not used",
            default=custom_value_default,
            subtype='COLOR' if len(custom_value_default) == 4 else 'NONE',
            size=len(custom_value_default)
        ))

    
    bpy.types.Scene.basecolor_suffix=bpy.props.StringProperty(
        name="Base Color Suffix",
        default="_basecolor",
        update=update_material_list
    )
    bpy.types.Scene.ao_suffix=bpy.props.StringProperty(
        name="AO Suffix",
        default="_ao",
    )
    bpy.types.Scene.normal_suffix=bpy.props.StringProperty(
        name="Normal Suffix",
        default="_normal",
    )
    bpy.types.Scene.roughness_suffix=bpy.props.StringProperty(
        name="Roughness Suffix",
        default="_roughness",
    )
    bpy.types.Scene.metallic_suffix=bpy.props.StringProperty(
        name="Metallic Suffix",
        default="_metallic",
    )
    bpy.types.Scene.opacity_suffix=bpy.props.StringProperty(
        name="Opacity Suffix",
        default="_opacity",
    )
    bpy.types.Scene.subsurface_suffix=bpy.props.StringProperty(
        name="Subsurface Suffix",
        default="_subsurface",
    )

    bpy.types.Scene.specular_suffix=bpy.props.StringProperty(
        name="Specular Suffix",
        default="_specular",
    )
    bpy.types.Scene.specular_tint_suffix=bpy.props.StringProperty(
        name="Specular Tint Suffix",
        default="_specular_tint",
    )
    bpy.types.Scene.anisotropic_suffix=bpy.props.StringProperty(
        name="Anisotropic Suffix",
        default="_anisotropic",
    )
    bpy.types.Scene.sheen_suffix=bpy.props.StringProperty(
        name="Sheen Suffix",
        default="_sheen",
    )
    bpy.types.Scene.sheen_tint_suffix=bpy.props.StringProperty(
        name="Sheen Tint Suffix",
        default="_sheen_tint",
    )
    bpy.types.Scene.clearcoat_suffix=bpy.props.StringProperty(
        name="Clearcoat Suffix",
        default="_clearcoat",
    )
    bpy.types.Scene.clearcoat_roughness_suffix=bpy.props.StringProperty(
        name="Clearcoat Roughness Suffix",
        default="_clearcoat_roughness",
    )
    bpy.types.Scene.ior_suffix=bpy.props.StringProperty(
        name="IOR Suffix",
        default="_ior",
    )
    bpy.types.Scene.transmission_suffix=bpy.props.StringProperty(
        name="Transmission Suffix",
        default="_transmission",
    )
    bpy.types.Scene.displacement_suffix=bpy.props.StringProperty(
        name="Displacement Suffix",
        default="_height",
    )
    bpy.types.Scene.emission_suffix=bpy.props.StringProperty(
        name="Emission Suffix",
        default="_emissive",
    )


    bpy.types.Scene.basecolor_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Base Color: ColorSpace",
        default="sRGB"
    )
    bpy.types.Scene.normal_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Normal: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.ao_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="AO: ColorSpace",
        default="sRGB"
    )
    bpy.types.Scene.roughness_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Roughness: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.metallic_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Metallic: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.opacity_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Opacity: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.subsurface_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Subsurface: ColorSpace",
        default="sRGB"
    )

    bpy.types.Scene.specular_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Specular: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.specular_tint_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Specular Tint: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.anisotropic_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Anisotropic: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.sheen_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Sheen: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.sheen_tint_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Sheen Tint: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.clearcoat_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Clearcoat: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.clearcoat_roughness_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Clearcoat Roughness: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.ior_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="IOR: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.transmission_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Transmission: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.displacement_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Displacement: ColorSpace",
        default="Non-Color"
    )
    bpy.types.Scene.emission_colorspace=bpy.props.EnumProperty(
        items=colorspace_items,
        name="Emission: ColorSpace",
        default="sRGB"
    )

    register_class(MATERIAL_OT_match_colorspaces)

    bpy.types.Scene.include_subfolders = bpy.props.BoolProperty(
        name="Include Subfolders",
        description="Search for textures in subfolders",
        default=False,
        update=update_material_list
    )

    bpy.types.Scene.preset_list = bpy.props.EnumProperty(
        items=update_preset_items,
        name="Presets",
        description="Select a preset to apply",
        update=on_preset_change
    )

    register_class(MATERIAL_UL_material_list)
    bpy.app.handlers.load_post.append(update_material_list)

def unregister():
    if update_material_list in bpy.app.handlers.load_post:
        bpy.app.handlers.load_post.remove(update_material_list)

    if load_handler in bpy.app.handlers.load_post:
        bpy.app.handlers.load_post.remove(load_handler)

    unregister_class(MaterialItem)
    del bpy.types.Scene.material_list
    del bpy.types.Scene.material_list_index

    del bpy.types.Scene.invert_normal_map_green_channel

    unregister_class(MATERIAL_OT_save_preset_dialog)
    unregister_class(MATERIAL_OT_delete_preset)
    unregister_class(MATERIAL_OT_dummy_operator)

    unregister_class(MATERIAL_OT_create_bsdf)
    unregister_class(MATERIAL_PT_panel)
    unregister_class(MATERIAL_UL_material_list)

    unregister_class(MATERIAL_OT_match_colorspaces)
    unregister_class(MATERIAL_OT_check_all)
    unregister_class(MATERIAL_OT_uncheck_all)
    unregister_class(MATERIAL_OT_invert_selection)



    for key in texture_properties.keys():
        delattr(bpy.types.Scene, f"{key}_suffix")
        delattr(bpy.types.Scene, f"{key}_colorspace")
        delattr(bpy.types.Scene, f"{key}_use_value")
        delattr(bpy.types.Scene, f"{key}_custom_value")


    del bpy.types.Scene.texture_folder_path
    del bpy.types.Scene.include_subfolders



if __name__=="__main__":
    register()