import bpy
from .register import register_class, unregister_class
from . import BAM_BamAutomat_M
from . import BAM_MaterialManager_M
from .translation.translate import GetTranslationDict




bl_info = {
    "name": "[BAM] AutoMat",
    "description": "Batch creation of materials from textures folder. Thanks KITBASH3D Team for early testing. Special thanks Nikita Kulchitskiy and Anderson Araujo",
    "version": (1, 0, 1),
    "blender": (4, 2, 0),
    "category": "3D View",
    "location": "View3D > right-side panel > AutoMat tab",
    "author": "AlexeyHRDesign,xz-blender repack",
    "tracker_url": "https://www.artstation.com/hrdesign",
    "warning": "",
}

class BAMAddonPreferences(bpy.types.AddonPreferences):
    bl_idname = __package__


    switch_to_cycles: bpy.props.BoolProperty(
        name="Switch to Cycles",
        description="Enable switching to Cycles renderer",
        default=False
    )
    
    texture_sync_interval: bpy.props.FloatProperty(
        name="Textures Sync Interval",
        description="How frequently to check for texture updates (in seconds).",
        default=8,
        min=0.1,
        max=300.0
    )

    def draw(self, context):
        layout = self.layout
        layout.prop(self, "switch_to_cycles")
        layout.prop(self, "texture_sync_interval")

def register():
    bpy.utils.register_class(BAMAddonPreferences)
    BAM_MaterialManager_M.register()
    BAM_BamAutomat_M.register()
    try:
        bpy.app.translations.register(__package__, GetTranslationDict())
    except Exception as e: print(e)

def unregister():
    bpy.utils.unregister_class(BAMAddonPreferences)
    BAM_MaterialManager_M.unregister()
    BAM_BamAutomat_M.unregister()
    try:
        bpy.app.translations.unregister(__package__)
    except Exception as e: print(e)

if __name__ == "__main__":
    register()
